/*
 *  Copyright (C) 2003 Ricardo Fernndez Pascual
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "galeon-action-refresh.h"
#include "galeon-shell.h"
#include "eel-gconf-extensions.h"
#include "prefs-strings.h"
#include "gul-gobject-misc.h"
#include "gul-toolbutton.h"
#include "gul-string.h"
#include "gul-gui.h"

#include <gtk/gtkstock.h>
#include <libgnome/gnome-i18n.h>
#include <string.h>

static void		galeon_action_refresh_init			(GaleonActionRefresh *action);
static void		galeon_action_refresh_class_init		(GaleonActionRefreshClass *class);
static void		galeon_action_refresh_finalize			(GObject *object);
static void		galeon_action_refresh_connect_proxy		(EggAction *action, GtkWidget *proxy);
static void		galeon_action_refresh_activate			(EggAction *action);

static EggActionClass *parent_class = NULL;


MAKE_GET_TYPE (galeon_action_refresh, "GaleonActionRefresh", GaleonActionRefresh,
	       galeon_action_refresh_class_init, 
	       galeon_action_refresh_init, GALEON_TYPE_ACTION);


static void
galeon_action_refresh_class_init (GaleonActionRefreshClass *class)
{
	GObjectClass *object_class = G_OBJECT_CLASS (class);
	EggActionClass *action_class = EGG_ACTION_CLASS (class);

	object_class->finalize = galeon_action_refresh_finalize;

	action_class->connect_proxy = galeon_action_refresh_connect_proxy;
	action_class->menu_item_type    = GTK_TYPE_IMAGE_MENU_ITEM;
	action_class->toolbar_item_type = GUL_TYPE_TOOLBUTTON;
	action_class->activate          = galeon_action_refresh_activate;

	parent_class = g_type_class_peek_parent (class);
}

static void
galeon_action_refresh_init (GaleonActionRefresh *action)
{
	g_object_set (G_OBJECT (action), "stock_id", GTK_STOCK_REFRESH, NULL);
	g_object_set (G_OBJECT (action), "tooltip", _("Display the latest content of the current page"), NULL);
}

static void
galeon_action_refresh_finalize (GObject *object)
{
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
galeon_action_refresh_activate (EggAction *a)
{
	GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (a));
	galeon_embed_reload (embed, EMBED_RELOAD_NORMAL);
}

static void
galeon_action_refresh_reload_all_tabs_activate_cb (GtkMenuItem *mi, GaleonActionRefresh *b)
{
	GList *tabs;
	GaleonWindow *window;
	GList *li;

	window = galeon_action_get_window (GALEON_ACTION (b));
	g_return_if_fail (window != NULL);

	tabs = galeon_window_get_tabs (window);
	for (li = tabs; li; li = li->next)
	{
		GaleonEmbed *embed = galeon_tab_get_embed (li->data);
		galeon_embed_reload (embed, EMBED_RELOAD_NORMAL);
	}
	
	g_list_free (tabs);
}

static void
galeon_action_refresh_reload_all_windows_activate_cb (GtkMenuItem *mi, GaleonActionRefresh *b)
{
	const GList *windows;
	const GList *lj;
	Session *session;

	session = galeon_shell_get_session (galeon_shell);
	windows = session_get_windows (session);

	for (lj = windows; lj; lj = lj->next)
	{
		GaleonWindow *window = lj->data;
		GList *tabs;
		GList *li;
		
		tabs = galeon_window_get_tabs (window);
		for (li = tabs; li; li = li->next)
		{
			GaleonEmbed *embed = galeon_tab_get_embed (li->data);
			galeon_embed_reload (embed, EMBED_RELOAD_NORMAL);
		}
		
		g_list_free (tabs);
	}
}

static void
galeon_action_refresh_normal_reload_activate_cb (GtkMenuItem *mi, GaleonActionRefresh *a)
{
	GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (a));
	galeon_embed_reload (embed, EMBED_RELOAD_NORMAL);
}

static void
galeon_action_refresh_bypass_cache_activate_cb (GtkMenuItem *mi, GaleonActionRefresh *a)
{
	GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (a));
	galeon_embed_reload (embed, EMBED_RELOAD_FORCE);
}

static gboolean
galeon_action_refresh_button_press_event_cb (GtkWidget *widget, GdkEventButton *event, 
					    GaleonActionRefresh *b)
{
	if (event->button == 1 && event->state & GDK_SHIFT_MASK)
	{
		GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (b));
		galeon_embed_reload (embed, EMBED_RELOAD_FORCE);

		return TRUE;
	}
	
	return FALSE;
}

static void
galeon_action_refresh_connect_proxy (EggAction *a, GtkWidget *proxy)
{
	GtkWidget *widget = proxy;

	if (GUL_IS_TOOLBUTTON (proxy))
	{
		GtkMenuShell *ms;
		widget = gul_toolbutton_get_button (GUL_TOOLBUTTON (proxy));

		ms = gul_toolbutton_get_menu (GUL_TOOLBUTTON (proxy));

		gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Normal _Reload"), 
			       G_CALLBACK (galeon_action_refresh_normal_reload_activate_cb), a);
		gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Bypass _Cache"), 
			       G_CALLBACK (galeon_action_refresh_bypass_cache_activate_cb), a);
		
		gul_gui_append_separator (GTK_WIDGET (ms));
		gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Reload All Tabs in This _Window"), 
			       G_CALLBACK (galeon_action_refresh_reload_all_tabs_activate_cb), a);
		gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Reload All Tabs in A_ll Windows"), 
			       G_CALLBACK (galeon_action_refresh_reload_all_windows_activate_cb), a);
	}

	g_signal_connect (widget, "button-press-event",
			  G_CALLBACK (galeon_action_refresh_button_press_event_cb), a);


	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (a, proxy);
}

